/* eslint global-require: off, no-console: off, promise/always-return: off */

/**
 * This module executes inside of electron's main process. You can start
 * electron renderer process from here and communicate with the other processes
 * through IPC.
 *
 * When running `npm run build` or `npm run build:main`, this file is compiled to
 * `./src/main.js` using webpack. This gives us some performance wins.
 */
import path from 'path';
import { app, BrowserWindow, shell, ipcMain, Tray, Menu } from 'electron';
import { autoUpdater } from 'electron-updater';
import log from 'electron-log';
import MenuBuilder from './menu';
import { resolveHtmlPath } from './util';
const nativeImage = require('electron').nativeImage;
const positioner = require('electron-traywindow-positioner');
const storage = require('electron-localstorage');
const fs = require('fs');
class AppUpdater {
    constructor() {
        log.transports.file.level = 'info';
        autoUpdater.logger = log;
        autoUpdater.checkForUpdatesAndNotify();
    }
}
let tray: Tray | null = null;
let mainWindow: BrowserWindow | null = null;
let siteWindown: BrowserWindow | null = null;
let homeWindown: BrowserWindow | null = null;

const isDev = process.env.NODE_ENV === 'development';
const isMac = process.platform === 'darwin';
const alignment = { x: 'left', y: 'up' };

const RESOURCES_PATH = app.isPackaged
    ? path.join(process.resourcesPath, 'assets')
    : path.join(__dirname, '../../assets');

const getAssetPath = (...paths: string[]): string => {
    return path.join(RESOURCES_PATH, ...paths);
};

let image: any;

if (isMac) {
    image = nativeImage.createFromPath(getAssetPath('/img/iconTemplate.png'));
} else {
    image = nativeImage.createFromPath(
        getAssetPath('/img/iconwinTemplate.png')
    );
}
Menu.setApplicationMenu(null);

const gotTheLock = app.requestSingleInstanceLock();
if (!gotTheLock) {
    app.quit();
}

app.setLoginItemSettings({
    openAtLogin: storage.getItem('Preferences')
        ? JSON.parse(storage.getItem('Preferences'))['selfStart']
        : true,
    openAsHidden: false,
    path: process.execPath,
    args: [],
});

ipcMain.on('createNewWindow-site', (event, arg) => {
    if (siteWindown) {
        siteWindown.focus();
        return;
    }
    siteWindown = new BrowserWindow({
        width: 620,
        height: 482,
        minWidth: 100,
        minHeight: 100,
        frame: true,
        resizable: false,
        webPreferences: {
            sandbox: false,
            nodeIntegration: true,
            contextIsolation: false,
            webSecurity: false,
        },
        // parent: siteWindown, //win是主窗口
    });
    siteWindown.loadURL(resolveHtmlPath(`index.html#/site`));
    isDev && siteWindown.webContents.openDevTools();
    siteWindown.on('closed', () => {
        siteWindown = null;
    });
});

ipcMain.on('resize-home', (event) => {
    if (!tray) return;
    mainWindow?.close();
    if (homeWindown) {
        homeWindown.focus();
        return;
    }
    homeWindown = new BrowserWindow({
        show: false,
        width: 350,
        height: 425,
        minWidth: 100,
        minHeight: 100,
        frame: false,
        resizable: false,
        minimizable: false,
        maximizable: false,
        webPreferences: {
            sandbox: false,
            nodeIntegration: true,
            contextIsolation: false,
            webSecurity: false,
        },
    });
    homeWindown.loadURL(resolveHtmlPath(`index.html#/home`));
    positioner.position(homeWindown, tray.getBounds(), alignment);
    isDev && homeWindown.webContents.openDevTools();
    // homeWindown.webContents.openDevTools();
    homeWindown.once('ready-to-show', () => {
        homeWindown?.show();
    });

    homeWindown.on('blur', () => {
        !isDev && homeWindown?.hide();
    });

    homeWindown.on('closed', () => {
        homeWindown = null;
    });
});

ipcMain.on('asynchronous-message', function (event, arg) {
    // arg是从渲染进程返回来的数据
    const USER_HOME = isMac
        ? (process.env.HOME || process.env.USERPROFILE) + '/bkunyun/'
        : (process.env.HOME || process.env.USERPROFILE) + '/.bkunyun/';
    fs.mkdir(USER_HOME, { recursive: true }, (err: any) => {
        if (err) return;
        fs.writeFile(
            USER_HOME + 'roco.json',
            JSON.stringify(arg),
            'utf8',
            (err: any) => {
                if (err) {
                    event.sender.send('asynchronous-reply', '写入失败');
                } else {
                    event.sender.send('asynchronous-reply', '写入成功');
                }
            }
        );
    });
});

ipcMain.on('window-close', (event) => {
    app.quit();
});

ipcMain.on('openAtLogin-true', () => {
    app.setLoginItemSettings({
        openAtLogin: true,
    });
});

ipcMain.on('openAtLogin-false', () => {
    app.setLoginItemSettings({
        openAtLogin: false,
    });
});

ipcMain.on('openLogin-windown', () => {
    mainWindow?.show();
    mainWindow?.focus();
});

if (process.env.NODE_ENV === 'production') {
    const sourceMapSupport = require('source-map-support');
    sourceMapSupport.install();
}

const isDebug =
    process.env.NODE_ENV === 'development' || process.env.DEBUG_PROD === 'true';

if (isDebug) {
    require('electron-debug')();
}

const installExtensions = async () => {
    const installer = require('electron-devtools-installer');
    const forceDownload = !!process.env.UPGRADE_EXTENSIONS;
    const extensions = ['REACT_DEVELOPER_TOOLS'];

    return installer
        .default(
            extensions.map((name) => installer[name]),
            forceDownload
        )
        .catch(console.log);
};

const createWindow = async () => {
    if (isDebug) {
        await installExtensions();
    }
    if (homeWindown) return;
    mainWindow = new BrowserWindow({
        show: false,
        width: 480,
        height: 486,
        minWidth: 100,
        minHeight: 100,
        icon: getAssetPath('icon.ico'),
        frame: true,
        resizable: false,
        titleBarStyle: 'hidden',
        maximizable: false,
        titleBarOverlay: {
            color: '#ffffff',
            height: 30,
        },
        webPreferences: {
            sandbox: false,
            nodeIntegration: true,
            contextIsolation: false,
            webSecurity: false,
        },
    });

    tray = new Tray(await image);
    const contextMenu = Menu.buildFromTemplate([
        {
            label: '退出',
            click: async () => {
                app.quit();
            },
        },
    ]);
    tray.setContextMenu(contextMenu);
    // tray.setTitle('test');
    tray.setToolTip('北鲲云');

    mainWindow.loadURL(resolveHtmlPath('index.html'));
    // mainWindow.webContents.openDevTools();
    mainWindow.once('ready-to-show', () => {
        if (!mainWindow) {
            throw new Error('"mainWindow" is not defined');
        }
        if (process.env.START_MINIMIZED) {
            mainWindow.minimize();
        } else {
            mainWindow.hide();
            // mainWindow.webContents.openDevTools();
        }
    });

    mainWindow.on('closed', () => {
        mainWindow = null;
    });

    // const menuBuilder = new MenuBuilder(mainWindow);
    // menuBuilder.buildMenu();

    // Open urls in the user's browser
    mainWindow.webContents.setWindowOpenHandler((edata) => {
        shell.openExternal(edata.url);
        return { action: 'deny' };
    });

    // Remove this if your app does not use auto updates
    // eslint-disable-next-line
    new AppUpdater();

    tray.on('click', function () {
        if (mainWindow) {
            mainWindow.show();
            return;
        }
        if (!homeWindown || !tray) return;
        homeWindown.show();
        positioner.position(homeWindown, tray.getBounds(), alignment);
    });

    tray.on('right-click', function () {
        if (!tray) return;
        tray.popUpContextMenu();
    });
};

/**
 * Add event listeners...
 */

app.on('window-all-closed', () => {
    // Respect the OSX convention of having the application in memory even
    // after all windows have been closed
    if (process.platform !== 'darwin') {
        app.quit();
    }
});

app.whenReady()
    .then(() => {
        createWindow();
        app.on('activate', () => {
            // On macOS it's common to re-create a window in the app when the
            // dock icon is clicked and there are no other windows open.
            if (mainWindow === null) createWindow();
        });
    })
    .catch(console.log);
